@@ -6,7 +6,6 @@ from typing import (
     Any,
     Callable,
     Generic,
-    Literal,
     Mapping,
     TypeGuard,
     TypeVar,
@@ -32,12 +31,16 @@ class Immutable:
 
 
 class BaseAction(Immutable):
-    type: str
+    ...
+
+
+class BaseEvent(Immutable):
+    ...
 
 
 # Type variables
 State = TypeVar('State', bound=Immutable)
-State_co = TypeVar('State_co', covariant=True)
+State_co = TypeVar('State_co', bound=Immutable, covariant=True)
 Action = TypeVar('Action', bound=BaseAction)
 SelectorOutput = TypeVar('SelectorOutput')
 SelectorOutput_co = TypeVar('SelectorOutput_co', covariant=True)
@@ -45,43 +48,42 @@ SelectorOutput_contra = TypeVar('SelectorOutput_contra', contravariant=True)
 ComparatorOutput = TypeVar('ComparatorOutput')
 Selector = Callable[[State], SelectorOutput]
 Comparator = Callable[[State], ComparatorOutput]
-SideEffect = Callable[[], None]
+Event = TypeVar('Event', bound=BaseEvent)
 
 
-class CompleteReducerResult(Immutable, Generic[State, Action]):
+class CompleteReducerResult(Immutable, Generic[State, Action, Event]):
     state: State
     actions: list[Action] | None = None
-    side_effects: list[SideEffect] | None = None
+    events: list[Event] | None = None
 
 
-ReducerResult = CompleteReducerResult[State, Action] | State
-
-
-ReducerType = Callable[[State | None, Action], ReducerResult[State, Action]]
+ReducerResult = CompleteReducerResult[State, Action, Event] | State
+ReducerType = Callable[[State | None, Action], ReducerResult[State, Action, Event]]
 AutorunReturnType = TypeVar('AutorunReturnType')
 AutorunReturnType_co = TypeVar('AutorunReturnType_co', covariant=True)
+EventSubscriber = Callable[[Event, Callable[[Event], None]], Callable[[], None]]
 
 
 class InitializationActionError(Exception):
     def __init__(self: InitializationActionError) -> None:
         super().__init__(
-            'The only accepted action type when state is None is "INIT"',
+            'The only accepted action type when state is None is "InitAction"',
         )
 
 
 class InitAction(BaseAction):
-    type: Literal['INIT'] = 'INIT'
+    ...
 
 
 class FinishAction(BaseAction):
-    type: Literal['FINISH'] = 'FINISH'
+    ...
 
 
 def is_reducer_result(
-    result: ReducerResult[State, Action],
-) -> TypeGuard[CompleteReducerResult[State, Action]]:
+    result: ReducerResult[State, Action, Event],
+) -> TypeGuard[CompleteReducerResult[State, Action, Event]]:
     return isinstance(result, CompleteReducerResult)
 
 
-def is_state(result: ReducerResult[State, Action]) -> TypeGuard[State]:
+def is_state(result: ReducerResult[State, Action, Event]) -> TypeGuard[State]:
     return not isinstance(result, CompleteReducerResult)
